﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;

namespace tpts.Logging
{
    /// <summary>
    /// Contract class for interface ILogEntryFilter
    /// </summary>
    [ContractClassFor(typeof(IFileRolloverStrategy))]
    internal abstract class IFileRolloverStrategyContract : IFileRolloverStrategy
    {

        /// <summary>
        /// Get the rolled file name based on the supplied file name.
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>        
        /// <remarks>
        /// Contract: pre  : fileName must not be null or empty or contain just whitespaces.
        ///           post : Result will not be null or empty or contain just whitespaces.
        /// </remarks>
        public string GetRolledFileName(string fileName)
        {
            Contract.Requires(!string.IsNullOrWhiteSpace(fileName));
            Contract.Requires(fileName != null);
            Contract.Requires(fileName.Length > 0);
            Contract.Ensures(!string.IsNullOrWhiteSpace(Contract.Result<string>()));
            Contract.Ensures(Contract.Result<string>() != null);
            Contract.Ensures(Contract.Result<string>().Length > 0);

            return default(string);
        }
    }
}
