﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;

namespace tpts.Logging
{
    /// <summary>
    /// Contract class for interface ILoggerContract
    /// </summary>
    [ContractClassFor(typeof(ILogger<>))]
    internal abstract class ILoggerContract<T> : ILogger<T>
    {

        /// <summary>
        /// Log the entry.
        /// </summary>
        /// <param name="entry"></param>
        /// <returns>A boolean indicating if the item was logged or not.</returns>
        /// <remarks>
        /// Contract: pre  : entry must not be null.
        ///           post : entry will be logged with a entered date/time of the current date/time.
        ///                  entry will be logged with a severity of information.
        ///                  entry will be logged with a category of blank.
        /// </remarks>
        public bool Log(T entry)
        {
            Contract.Requires(entry != null);

            return default(bool);
        }

        /// <summary>
        /// Log the entry.
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="category"></param>
        /// <returns>A boolean indicating if the item was logged or not.</returns>
        /// <remarks>
        /// Contract: pre  : entry must not be null.
        ///                  category must not be null.
        ///           post : entry will be logged with a entered date/time of the current date/time.
        ///                  entry will be logged with a severity of information.
        /// </remarks>
        public bool Log(T entry, string category)
        {
            Contract.Requires(entry != null);
            Contract.Requires(category != null);

            return default(bool);
        }

        /// <summary>
        /// Log the entry.
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="severity"></param>
        /// <returns>A boolean indicating if the item was logged or not.</returns>
        /// <remarks>
        /// Contract: pre  : entry must not be null.
        ///           post : entry will be logged with a entered date/time of the current date/time.
        ///                  entry will be logged with a category of blank.
        /// </remarks>
        public bool Log(T entry, LogEntrySeverity severity)
        {
            Contract.Requires(entry != null);

            return default(bool);
        }

        /// <summary>
        /// Log the entry.
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="severity"></param>
        /// <param name="category"></param>
        /// <returns>A boolean indicating if the item was logged or not.</returns>
        /// <remarks>
        /// Contract: pre  : entry must not be null.
        ///                  category must not be null.
        ///           post : entry will be logged with a entered date/time of the current date/time.
        /// </remarks>
        public bool Log(T entry, LogEntrySeverity severity, string category)
        {
            Contract.Requires(entry != null);
            Contract.Requires(category != null);

            return default(bool);
        }

        public ILogEntryFormatter<T> LogEntryFormatter
        {
            set { throw new NotImplementedException(); }
        }

        public ILogEntryFilter<T> LogEntryFilter
        {
            set { throw new NotImplementedException(); }
        }

    }
}
