﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;

namespace tpts.Logging
{
    /// <summary>
    /// Class to filter log entries where the log entry severity is greater than or equal to the specified threshold.
    /// </summary>
    /// <typeparam name="T">The type of item logged.</typeparam>
    /// <remarks>
    /// Class is immutable.
    /// </remarks>
    public class SeverityFilter<T>: ILogEntryFilter<T>
    {

        #region Declarations

        private readonly LogEntrySeverity _SeverityThreshold;

        #endregion

        #region Constructors

        /// <summary>
        /// Constructor for SeverityFilter, specifying the severity threshold.
        /// </summary>
        /// <param name="severityThreshold"></param>
        public SeverityFilter(LogEntrySeverity severityThreshold)
        {
            _SeverityThreshold = severityThreshold;
        }

        #endregion

        #region Properties 

        /// <summary>
        /// Get/Set the severity threshold.
        /// </summary>
        protected virtual LogEntrySeverity SeverityThreshold
        {
            get { return _SeverityThreshold; }
        }

        #endregion

        #region Public Methods
        #endregion

        #region Protected Methods
        #endregion

        #region Private Methods
        #endregion

        #region ILogEntryFilter

        /// <summary>
        /// Return true if the log entry severity is greater than or equal to the filter severity threshold, otherwise false.
        /// </summary>
        /// <param name="logEntry"></param>
        /// <returns>A boolean indicating if the log entry severity is greater than or equal to the filter severity threshold.</returns>
        /// <remarks>
        /// Contract: pre  : logEntry must not be null.
        /// </remarks>
        public virtual bool Accept(LogEntry<T> logEntry)
        {
            return logEntry.Severity >= SeverityThreshold;
        }

        #endregion
    
    }
}
