﻿using tpts.Logging;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Diagnostics.Contracts;

namespace tptsLoggingUnitTests
{


    /// <summary>
    ///This is a test class for CategoryFilterTest and is intended
    ///to contain all CategoryFilterTest Unit Tests
    ///</summary>
    [TestClass()]
    public class CategoryFilterTest
    {


        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion


        /// <summary>
        /// Constructor for LogEntryTest
        /// </summary>
        public CategoryFilterTest()
        {
            // Add event handler for contract failures, required so tests can trap any contract exceptions
            // Required to allow unit tests to test contracts are correct.
            Contract.ContractFailed += (sender, e) =>
            {
                e.SetHandled();
                e.SetUnwind();
                throw new ContractTestFailedException(e.FailureKind + ": " + e.Message);
            };
        }

        #region Test Constructors

        [TestMethod()]
        public void ConstructorTest1()
        {
            string category = "ConstructorTest1 Category";
            ConstructorTestAsString(category);
        }

        [TestMethod()]
        public void ConstructorTest2()
        {
            string category = "";
            ConstructorTestAsString(category);
        }

        [TestMethod()]
        public void ConstructorTest3()
        {
            string category = "ConstructorTest1 Category";
            ConstructorTestAsLogEntryType(category);
        }

        [TestMethod()]
        public void ConstructorTest4()
        {
            string category = "";
            ConstructorTestAsLogEntryType(category);
        }
        #endregion

        #region Test Methods

        /// <summary>
        ///A test for Accept - test categories match, both non blank.
        ///</summary>
        [TestMethod()]
        public void AcceptTest1()
        {
            string category = "C";
            string logEntryCategory = category;
            AcceptTestAsString(category, logEntryCategory, true);
        }

        /// <summary>
        ///A test for Accept - test categories do not match, filter and log entry categories non blank.
        ///</summary>
        [TestMethod()]
        public void AcceptTest2()
        {
            string category = "C";
            string logEntryCategory = "NoMatch" + category;
            AcceptTestAsString(category, logEntryCategory, false);
        }

        /// <summary>
        ///A test for Accept - test categories do not match, filter category non blank, log entry category blank.
        ///</summary>
        [TestMethod()]
        public void AcceptTest3()
        {
            string category = "C";
            string logEntryCategory = "";
            AcceptTestAsString(category, logEntryCategory, false);
        }

        /// <summary>
        ///A test for Accept - test categories match, filter category blank, log entry category non blank.
        ///</summary>
        [TestMethod()]
        public void AcceptTest4()
        {
            string category = "";
            string logEntryCategory = "NoMatch" + category;
            AcceptTestAsString(category, logEntryCategory, true);
        }

        /// <summary>
        ///A test for Accept - test categories match, filter and log entry categories both blank.
        ///</summary>
        [TestMethod()]
        public void AcceptTest5()
        {
            string category = "";
            string logEntryCategory = "";
            AcceptTestAsString(category, logEntryCategory, true);
        }

        /// <summary>
        ///A test for Accept - test categories match, both non blank.
        ///</summary>
        [TestMethod()]
        public void AcceptTest6()
        {
            string category = "C";
            string logEntryCategory = category;
            AcceptTestAsLogEntryType(category, logEntryCategory, true);
        }

        /// <summary>
        ///A test for Accept - test categories do not match, filter and log entry categories non blank.
        ///</summary>
        [TestMethod()]
        public void AcceptTest7()
        {
            string category = "C";
            string logEntryCategory = "NoMatch" + category;
            AcceptTestAsLogEntryType(category, logEntryCategory, false);
        }

        /// <summary>
        ///A test for Accept - test categories do not match, filter category non blank, log entry category blank.
        ///</summary>
        [TestMethod()]
        public void AcceptTest8()
        {
            string category = "C";
            string logEntryCategory = "";
            AcceptTestAsLogEntryType(category, logEntryCategory, false);
        }

        /// <summary>
        ///A test for Accept - test categories match, filter category blank, log entry category non blank.
        ///</summary>
        [TestMethod()]
        public void AcceptTest9()
        {
            string category = "";
            string logEntryCategory = "NoMatch" + category;
            AcceptTestAsLogEntryType(category, logEntryCategory, true);
        }

        /// <summary>
        ///A test for Accept - test categories match, filter and log entry categories both blank.
        ///</summary>
        [TestMethod()]
        public void AcceptTest10()
        {
            string category = "";
            string logEntryCategory = "";
            AcceptTestAsLogEntryType(category, logEntryCategory, true);
        }

        #endregion

        #region Test Properties

        #endregion

        #region Contract Tests

        /// <summary>
        /// Test contract volilation when passing in null category in constructor.
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void CategoryFilterAcceptContractTest()
        {
            try
            {
                LogEntry<string> logEntry = null;
                string category = "FilterCategory";
                CategoryFilter<string> target = new CategoryFilter<string>(category);
                bool actual;
                actual = target.Accept(logEntry);
            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: CategoryFilterAcceptContractTest", ex);
                }
                else
                {
                    throw;
                }
            }

        }

        #endregion

        #region Private Methods
        /// <summary>
        ///A test for Accept - test categories match, both non blank.
        ///</summary>
        public void AcceptTestAsString(string category, string logEntryCategory, bool expected)
        {
            CategoryFilter<string> target = new CategoryFilter<string>(category);
            LogEntry<string> logEntry = new LogEntry<string>(DateTime.Now, "Test Entry", LogEntrySeverity.Error, logEntryCategory);
            bool actual;
            actual = target.Accept(logEntry);
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for Accept - test categories match, both non blank.
        ///</summary>
        public void AcceptTestAsLogEntryType(string category, string logEntryCategory, bool expected)
        {
            LogEntryType leType = new LogEntryType();
            CategoryFilter<LogEntryType> target = new CategoryFilter<LogEntryType>(category);
            LogEntry<LogEntryType> logEntry = new LogEntry<LogEntryType>(DateTime.Now, leType, LogEntrySeverity.Error, logEntryCategory);
            bool actual;
            actual = target.Accept(logEntry);
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        /// Test constructor
        /// </summary>
        public void ConstructorTestAsString(string category)
        {
            CategoryFilter<string> categoryFilter = new CategoryFilter<string>(category);
            string actual;
            actual = categoryFilter.Category;
            Assert.AreEqual(category, actual);
        }

        /// <summary>
        /// Test constructor
        /// </summary>
        public void ConstructorTestAsLogEntryType(string category)
        {
            CategoryFilter<LogEntryType> categoryFilter = new CategoryFilter<LogEntryType>(category);
            string actual;
            actual = categoryFilter.Category;
            Assert.AreEqual(category, actual);
        }

        #endregion
    }
}
