﻿using tpts.Logging;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Diagnostics.Contracts;

namespace tptsLoggingUnitTests
{


    /// <summary>
    ///This is a test class for CompositeLoggerTest and is intended
    ///to contain all CompositeLoggerTest Unit Tests
    ///</summary>
    [TestClass()]
    public class CompositeLoggerTest
    {


        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion

        /// <summary>
        /// Constructor for ConsoleLoggerTest
        /// </summary>
        public CompositeLoggerTest()
        {
            // Add event handler for contract failures, required so tests can trap any contract exceptions
            // Required to allow unit tests to test contracts are correct.
            Contract.ContractFailed += (sender, e) =>
            {
                e.SetHandled();
                e.SetUnwind();
                throw new ContractTestFailedException(e.FailureKind + ": " + e.Message);
            };
        }

        /// <summary>
        ///A test for CompositeLogger`1 Constructor
        ///</summary>
        public void CompositeLoggerConstructorTestHelper<T>()
        {
            CompositeLogger<T> target = new CompositeLogger<T>();
            Assert.AreEqual(0, target.Count);
        }

        [TestMethod()]
        public void CompositeLoggerConstructorTest()
        {
            CompositeLoggerConstructorTestHelper<GenericParameterHelper>();
        }

        /// <summary>
        ///A test for Add
        ///</summary>
        public void AddTestHelper<T>()
        {
            CompositeLogger<T> target = new CompositeLogger<T>();
            GenericLogger<T> logger = new GenericLogger<T>();
            bool expected = true;
            bool actual;
            actual = target.Add(logger);
            Assert.AreEqual(expected, actual, "First Logger Add");

            expected = false;
            actual = target.Add(logger);
            Assert.AreEqual(expected, actual, "Same Logger Add");

            logger = new GenericLogger<T>();
            expected = true;
            actual = target.Add(logger);
            Assert.AreEqual(expected, actual, "Different Logger Add");

        }

        [TestMethod()]
        public void AddTest()
        {
            AddTestHelper<GenericParameterHelper>();
        }

        /// <summary>
        ///A test for Clear
        ///</summary>
        public void ClearTestHelper<T>()
        {
            CompositeLogger<T> target = new CompositeLogger<T>();
            Random r = new Random();
            int itemCount = r.Next(10, 100);
            for (int index = 1; index <= itemCount; index++)
            {
                GenericLogger<T> logger = new GenericLogger<T>();
                target.Add(logger);
            }
            Assert.AreEqual(itemCount, target.Count);
            target.Clear();
            Assert.AreEqual(0, target.Count);
        }

        [TestMethod()]
        public void ClearTest()
        {
            ClearTestHelper<GenericParameterHelper>();
        }

        /// <summary>
        ///A test for Contains
        ///</summary>
        public void ContainsTestHelper<T>()
        {
            CompositeLogger<T> target = new CompositeLogger<T>();
            GenericLogger<T> logger1 = new GenericLogger<T>();
            GenericLogger<T> logger2 = new GenericLogger<T>();
            GenericLogger<T> logger3 = new GenericLogger<T>();
            GenericLogger<T> logger4 = new GenericLogger<T>();

            bool actual;

            actual = target.Add(logger1);
            Assert.IsTrue(actual);
            actual = target.Add(logger2);
            Assert.IsTrue(actual);
            actual = target.Add(logger3);
            Assert.IsTrue(actual);
            actual = target.Add(logger4);
            Assert.IsTrue(actual);

            Assert.AreEqual(4, target.Count);

            bool expected = true;
            actual = target.Contains(logger1);
            Assert.AreEqual(expected, actual);
            actual = target.Contains(logger2);
            Assert.AreEqual(expected, actual);
            actual = target.Contains(logger3);
            Assert.AreEqual(expected, actual);
            actual = target.Contains(logger4);
            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void ContainsTest()
        {
            ContainsTestHelper<GenericParameterHelper>();
        }

        /// <summary>
        ///A test for Remove
        ///</summary>
        public void RemoveTestHelper<T>()
        {
            CompositeLogger<T> target = new CompositeLogger<T>();
            GenericLogger<T> logger1 = new GenericLogger<T>();
            GenericLogger<T> logger2 = new GenericLogger<T>();
            GenericLogger<T> logger3 = new GenericLogger<T>();
            GenericLogger<T> logger4 = new GenericLogger<T>();

            bool actual;

            actual = target.Add(logger1);
            Assert.IsTrue(actual);
            actual = target.Add(logger2);
            Assert.IsTrue(actual);
            actual = target.Add(logger3);
            Assert.IsTrue(actual);
            actual = target.Add(logger4);
            Assert.IsTrue(actual);

            Assert.AreEqual(4, target.Count);

            target.Remove(logger2);
            Assert.AreEqual(3, target.Count);
        }

        [TestMethod()]
        public void RemoveTest()
        {
            RemoveTestHelper<GenericParameterHelper>();
        }

        /// <summary>
        ///A test for Count
        ///</summary>
        public void CountTestHelper<T>()
        {
            CompositeLogger<T> target = new CompositeLogger<T>();
            Random r = new Random();
            int itemCount = r.Next(10, 100);
            for (int index = 1; index <= itemCount; index++)
            {
                GenericLogger<T> logger = new GenericLogger<T>();
                target.Add(logger);
            }
            Assert.AreEqual(itemCount, target.Count);
        }

        [TestMethod()]
        public void CountTest()
        {
            CountTestHelper<GenericParameterHelper>();
        }

        [TestMethod()]
        public void LogTest()
        {
            CompositeLogger<string> target = new CompositeLogger<string>();
            GenericLogger<string> logger1 = new GenericLogger<string>();
            logger1.LoggerName = "Logger1";
            GenericLogger<string> logger2 = new GenericLogger<string>();
            logger2.LoggerName = "Logger2";
            GenericLogger<string> logger3 = new GenericLogger<string>();
            logger3.LoggerName = "Logger3";
            GenericLogger<string> logger4 = new GenericLogger<string>();
            logger4.LoggerName = "Logger4";

            bool actual;

            actual = target.Add(logger1);
            Assert.IsTrue(actual);
            actual = target.Add(logger2);
            Assert.IsTrue(actual);
            actual = target.Add(logger3);
            Assert.IsTrue(actual);
            actual = target.Add(logger4);
            Assert.IsTrue(actual);

            Assert.AreEqual(4, target.Count);

            Assert.AreEqual(0, logger1.LoggedCount);
            Assert.AreEqual(0, logger2.LoggedCount);
            Assert.AreEqual(0, logger3.LoggedCount);
            Assert.AreEqual(0, logger4.LoggedCount);

            Random r = new Random();
            int loggedCount = r.Next(10, 100);

            for (int index = 1; index <= loggedCount; index++)
            {
                actual = target.Log("Test Entry");
                Assert.AreEqual(true, actual);
            }

            Assert.AreEqual(loggedCount, logger1.LoggedCount,"Logger1");
            Assert.AreEqual(loggedCount, logger2.LoggedCount, "Logger2");
            Assert.AreEqual(loggedCount, logger3.LoggedCount, "Logger3");
            Assert.AreEqual(loggedCount, logger4.LoggedCount, "Logger4");

        }

        #region Standard Logger Tests

        [TestMethod()]
        public void StandardLoggerTests()
        {
            ILoggerTestHelper loggerTests = new ILoggerTestHelper();
            loggerTests.RunStandardTests(CreateCompositeLogger<string>());
        }

        #endregion

        #region Contract Tests

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest1()
        {
            try
            {
                CompositeLogger<string> logger = CreateCompositeLogger<string>();
                string entryText = null;

                logger.Log(entryText);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest1", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest2()
        {
            try
            {
                CompositeLogger<string> logger = CreateCompositeLogger<string>();
                string entryText = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;

                logger.Log(entryText, severity);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest2", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest3()
        {
            try
            {
                CompositeLogger<string> logger = CreateCompositeLogger<string>();
                string entryText = null;
                string category = "LogNullEntryContractTest3 Category";

                logger.Log(entryText, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest3", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest4()
        {
            try
            {
                CompositeLogger<string> logger = CreateCompositeLogger<string>();
                string entryText = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;
                string category = "LogNullEntryContractTest4 Category";

                logger.Log(entryText, severity, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest4", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null category
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullCategoryContractTest1()
        {
            try
            {
                CompositeLogger<string> logger = CreateCompositeLogger<string>();
                string entryText = "LogNullCategoryContractTest1 Entry";
                string category = null;

                logger.Log(entryText, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullCategoryContractTest1", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullCategoryContractTest2()
        {
            try
            {
                CompositeLogger<string> logger = CreateCompositeLogger<string>();
                string entryText = "LogNullCategoryContractTest2 Entry";
                string category = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;

                logger.Log(entryText, severity, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullCategoryContractTest2", ex);
                }
                else
                {
                    throw;
                }
            }
        }
        #endregion

        #region Private Methods

        private CompositeLogger<T> CreateCompositeLogger<T>()
        {
            CompositeLogger<T> target = new CompositeLogger<T>();
            GenericLogger<T> logger1 = new GenericLogger<T>();
            GenericLogger<T> logger2 = new GenericLogger<T>();
            GenericLogger<T> logger3 = new GenericLogger<T>();
            GenericLogger<T> logger4 = new GenericLogger<T>();

            bool actual;

            actual = target.Add(logger1);
            Assert.IsTrue(actual);
            actual = target.Add(logger2);
            Assert.IsTrue(actual);
            actual = target.Add(logger3);
            Assert.IsTrue(actual);
            actual = target.Add(logger4);
            Assert.IsTrue(actual);

            Assert.AreEqual(4, target.Count);

            return target;
        }

        #endregion

    }
}
