﻿using tpts.Logging;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Diagnostics.Contracts;

namespace tptsLoggingUnitTests
{


    /// <summary>
    ///This is a test class for ConsoleLoggerTest and is intended
    ///to contain all ConsoleLoggerTest Unit Tests
    ///</summary>
    [TestClass()]
    public class ConsoleLoggerTest
    {

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion


        /// <summary>
        /// Constructor for ConsoleLoggerTest
        /// </summary>
        public ConsoleLoggerTest()
        {
            // Add event handler for contract failures, required so tests can trap any contract exceptions
            // Required to allow unit tests to test contracts are correct.
            Contract.ContractFailed += (sender, e) =>
            {
                e.SetHandled();
                e.SetUnwind();
                throw new ContractTestFailedException(e.FailureKind + ": " + e.Message);
            };
        }

        #region Standard Logger Tests

        [TestMethod()]
        public void StandardLoggerTests()
        {
            ILoggerTestHelper loggerTests = new ILoggerTestHelper();
            loggerTests.RunStandardTests(CreateConsoleLogger<string>());
        }


        #endregion

        #region Contract Tests

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest1()
        {
            try
            {
                ConsoleLogger<string> logger = CreateConsoleLogger<string>();
                string entryText = null;

                logger.Log(entryText);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest1", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest2()
        {
            try
            {
                ConsoleLogger<string> logger = CreateConsoleLogger<string>();
                string entryText = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;

                logger.Log(entryText, severity);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest2", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest3()
        {
            try
            {
                ConsoleLogger<string> logger = CreateConsoleLogger<string>();
                string entryText = null;
                string category = "LogNullEntryContractTest3 Category";

                logger.Log(entryText, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest3", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest4()
        {
            try
            {
                ConsoleLogger<string> logger = CreateConsoleLogger<string>();
                string entryText = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;
                string category = "LogNullEntryContractTest4 Category";

                logger.Log(entryText, severity, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest4", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null category
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullCategoryContractTest1()
        {
            try
            {
                ConsoleLogger<string> logger = CreateConsoleLogger<string>();
                string entryText = "LogNullCategoryContractTest1 Entry";
                string category = null;

                logger.Log(entryText, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullCategoryContractTest1", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullCategoryContractTest2()
        {
            try
            {
                ConsoleLogger<string> logger = CreateConsoleLogger<string>();
                string entryText = "LogNullCategoryContractTest2 Entry";
                string category = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;

                logger.Log(entryText, severity, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullCategoryContractTest2", ex);
                }
                else
                {
                    throw;
                }
            }
        }
        #endregion


        #region Private Methods

        private ConsoleLogger<T> CreateConsoleLogger<T>()
        {
            return new ConsoleLogger<T>();
        }

        #endregion

    }
}
