﻿using tpts.Logging;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Diagnostics.Contracts;

namespace tptsLoggingUnitTests
{


    /// <summary>
    ///This is a test class for FileLoggerTest and is intended
    ///to contain all FileLoggerTest Unit Tests
    ///</summary>
    [TestClass()]
    public class FileLoggerTest
    {

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion

        /// <summary>
        /// Constructor for EventLogLoggerTest
        /// </summary>
        public FileLoggerTest()
        {
            // Add event handler for contract failures, required so tests can trap any contract exceptions
            // Required to allow unit tests to test contracts are correct.
            Contract.ContractFailed += (sender, e) =>
            {
                e.SetHandled();
                e.SetUnwind();
                throw new ContractTestFailedException(e.FailureKind + ": " + e.Message);
            };
        }

        #region Rollover Tests


        /// <summary>
        /// Test rolling of file name with no path and extension.
        /// </summary>
        [TestMethod()]
        public void RollOverTest1()
        {

            string logFileName = "RollOverTest1.txt";
            IFileRolloverStrategy rolloverStrategy = new FixedDateRolloverStrategy();
            FileLogger<string> logger = CreateFileLogger<string>(logFileName, rolloverStrategy);
            string expected = rolloverStrategy.GetRolledFileName(logFileName);
            string entryText = "RollOverTest1 Entry";
            if (System.IO.File.Exists(expected))
            {
                System.IO.File.Delete(expected);
            }
            Assert.IsFalse(System.IO.File.Exists(expected));
            logger.Log(entryText);
            Assert.IsTrue(System.IO.File.Exists(expected), "Log File '" + expected + "' was not found");
            if (System.IO.File.Exists(expected))
            {
                System.IO.File.Delete(expected);
            }
        }

        /// <summary>
        /// Test rolling of file name with path and extension.
        /// </summary>
        [TestMethod()]
        public void RollOverTest2()
        {

            string logFileName = System.IO.Path.Combine(System.IO.Path.GetTempPath(), "RollOverTest2.txt");
            IFileRolloverStrategy rolloverStrategy = new FixedDateRolloverStrategy();
            FileLogger<string> logger = CreateFileLogger<string>(logFileName, rolloverStrategy);
            string expected = rolloverStrategy.GetRolledFileName(logFileName);
            string entryText = "RollOverTest2 Entry";
            if (System.IO.File.Exists(expected))
            {
                System.IO.File.Delete(expected);
            }
            Assert.IsFalse(System.IO.File.Exists(expected));
            logger.Log(entryText);
            Assert.IsTrue(System.IO.File.Exists(expected), "Log File '" + expected + "' was not found");
            if (System.IO.File.Exists(expected))
            {
                System.IO.File.Delete(expected);
            }
        }

        /// <summary>
        /// Test rolling of file name with no path and no extension.
        /// </summary>
        [TestMethod()]
        public void RollOverTest3()
        {

            string logFileName = "RollOverTest3.txt";
            IFileRolloverStrategy rolloverStrategy = new FixedDateRolloverStrategy();
            FileLogger<string> logger = CreateFileLogger<string>(logFileName, rolloverStrategy);
            string expected = rolloverStrategy.GetRolledFileName(logFileName);
            string entryText = "RollOverTest3 Entry";
            if (System.IO.File.Exists(expected))
            {
                System.IO.File.Delete(expected);
            }
            Assert.IsFalse(System.IO.File.Exists(expected));
            logger.Log(entryText);
            Assert.IsTrue(System.IO.File.Exists(expected), "Log File '" + expected + "' was not found");
            if (System.IO.File.Exists(expected))
            {
                System.IO.File.Delete(expected);
            }
        }

        /// <summary>
        /// Test rolling of file name with path and no extension.
        /// </summary>
        [TestMethod()]
        public void RollOverTest4()
        {

            string logFileName = System.IO.Path.Combine(System.IO.Path.GetTempPath(), "RollOverTest4");
            IFileRolloverStrategy rolloverStrategy = new FixedDateRolloverStrategy();
            FileLogger<string> logger = CreateFileLogger<string>(logFileName, rolloverStrategy);
            string expected = rolloverStrategy.GetRolledFileName(logFileName);
            string entryText = "RollOverTest4 Entry";
            if (System.IO.File.Exists(expected))
            {
                System.IO.File.Delete(expected);
            }
            Assert.IsFalse(System.IO.File.Exists(expected));
            logger.Log(entryText);
            Assert.IsTrue(System.IO.File.Exists(expected), "Log File '" + expected + "' was not found");
            if (System.IO.File.Exists(expected))
            {
                System.IO.File.Delete(expected);
            }
        }

        #endregion

        #region File Creation Tests

        [TestMethod()]
        public void FileCreationTest1()
        {
            string logFileName = "FileCreationTest1.txt";
            if (System.IO.File.Exists(logFileName))
            {
                System.IO.File.Delete(logFileName);
            }
            FileLogger<string> logger = CreateFileLogger<string>(logFileName);
            FixedFormatter<string> formatter = new FixedFormatter<string>();
            logger.LogEntryFormatter = formatter;
            string entryText = "FileCreationTest1 Text";
            logger.Log(entryText);
            bool expected = true;
            bool actual;
            actual = System.IO.File.Exists(logFileName);
            Assert.AreEqual(expected, actual, "Log File was not created");
            if (actual)
            {
                string fileContents = System.IO.File.ReadAllText(logFileName);
                actual = fileContents.Contains(entryText);
                Assert.AreEqual(expected, actual, "Log File does not contain the log entry.");
            }
        }

        [TestMethod()]
        public void FileCreationTest2()
        {
            string logFileName = System.IO.Path.Combine(System.IO.Path.GetTempPath(), "FileCreationTest2.txt");
            if (System.IO.File.Exists(logFileName))
            {
                System.IO.File.Delete(logFileName);
            }
            FileLogger<string> logger = CreateFileLogger<string>(logFileName);
            FixedFormatter<string> formatter = new FixedFormatter<string>();
            logger.LogEntryFormatter = formatter;
            string entryText = "FileCreationTest2 Text";
            logger.Log(entryText);
            bool expected = true;
            bool actual;
            actual = System.IO.File.Exists(logFileName);
            Assert.AreEqual(expected, actual, "Log File was not created");
            if (actual)
            {
                string fileContents = System.IO.File.ReadAllText(logFileName);
                actual = fileContents.Contains(entryText);
                Assert.AreEqual(expected, actual, "Log File does not contain the log entry.");
            }
        }
        #endregion

        #region Standard Logger Tests

        [TestMethod()]
        public void StandardLoggerTests()
        {
            ILoggerTestHelper loggerTests = new ILoggerTestHelper();
            string logFileName = "StandardLoggerTests.txt";
            loggerTests.RunStandardTests(CreateFileLogger<string>(logFileName));
        }


        #endregion

        #region Contract Tests

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest1()
        {
            try
            {
                string logFileName = "LogNullEntryContractTest1.txt";
                FileLogger<string> logger = CreateFileLogger<string>(logFileName);
                string entryText = null;

                logger.Log(entryText);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest1", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest2()
        {
            try
            {
                string logFileName = "LogNullEntryContractTest2.txt";
                FileLogger<string> logger = CreateFileLogger<string>(logFileName);
                string entryText = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;

                logger.Log(entryText, severity);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest2", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest3()
        {
            try
            {
                string logFileName = "LogNullEntryContractTest3.txt";
                FileLogger<string> logger = CreateFileLogger<string>(logFileName);
                string entryText = null;
                string category = "LogNullEntryContractTest3 Category";

                logger.Log(entryText, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest3", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest4()
        {
            try
            {
                string logFileName = "LogNullEntryContractTest4.txt";
                FileLogger<string> logger = CreateFileLogger<string>(logFileName);
                string entryText = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;
                string category = "LogNullEntryContractTest4 Category";

                logger.Log(entryText, severity, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest4", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null category
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullCategoryContractTest1()
        {
            try
            {
                string logFileName = "LogNullCategoryContractTest1.txt";
                FileLogger<string> logger = CreateFileLogger<string>(logFileName);
                string entryText = "LogNullCategoryContractTest1 Entry";
                string category = null;

                logger.Log(entryText, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullCategoryContractTest1", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullCategoryContractTest2()
        {
            try
            {
                string logFileName = "LogNullCategoryContractTest2.txt";
                FileLogger<string> logger = CreateFileLogger<string>(logFileName);
                string entryText = "LogNullCategoryContractTest2 Entry";
                string category = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;

                logger.Log(entryText, severity, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullCategoryContractTest2", ex);
                }
                else
                {
                    throw;
                }
            }
        }
        #endregion

        #region Private Methods

        private FileLogger<T> CreateFileLogger<T>(string logFileName)
        {
            return new FileLogger<T>(logFileName);
        }

        private FileLogger<T> CreateFileLogger<T>(string logFileName, IFileRolloverStrategy rolloverStrategy)
        {
            return new FileLogger<T>(logFileName, rolloverStrategy);
        }

        #endregion

    }
}
