﻿using tpts.Logging;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Diagnostics.Contracts;

namespace tptsLoggingUnitTests
{


    /// <summary>
    ///This is a test class for FileSizeRolloverStrategyTest and is intended
    ///to contain all FileSizeRolloverStrategyTest Unit Tests
    ///</summary>
    [TestClass()]
    public class FileSizeRolloverStrategyTest
    {


        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion

               /// <summary>
        /// Constructor for ConsoleLoggerTest
        /// </summary>
        public FileSizeRolloverStrategyTest()
        {
            // Add event handler for contract failures, required so tests can trap any contract exceptions
            // Required to allow unit tests to test contracts are correct.
            Contract.ContractFailed += (sender, e) =>
            {
                e.SetHandled();
                e.SetUnwind();
                throw new ContractTestFailedException(e.FailureKind + ": " + e.Message);
            };
        }


        #region Methods Test

        /// <summary>
        ///A test for GetRolledFileName - max size > 0 rollover - file name and extension.
        ///</summary>
        [TestMethod()]
        public void GetRolledFileNameTest5()
        {
            int maximumFileSize = 1;
            FileSizeRolloverStrategy target = new FileSizeRolloverStrategy(maximumFileSize);
            string fileName = "GetRolledFileNameTest5.txt";
            if (System.IO.File.Exists(fileName))
            {
                System.IO.File.Delete(fileName);
            }
            if (System.IO.File.Exists(AppendFileCounter(fileName, 1)))
            {
                System.IO.File.Delete(AppendFileCounter(fileName, 1));
            }
            CreateFileWithSize(fileName, maximumFileSize * 2);
            string expected = AppendFileCounter(fileName,1);
            string actual;
            actual = target.GetRolledFileName(fileName);
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for GetRolledFileName - max size > 0 rollover - file name and no extension.
        ///</summary>
        [TestMethod()]
        public void GetRolledFileNameTest6()
        {
            int maximumFileSize = 1;
            FileSizeRolloverStrategy target = new FileSizeRolloverStrategy(maximumFileSize);
            string fileName = "GetRolledFileNameTest6";
            if (System.IO.File.Exists(fileName))
            {
                System.IO.File.Delete(fileName);
            }
            if (System.IO.File.Exists(AppendFileCounter(fileName, 1)))
            {
                System.IO.File.Delete(AppendFileCounter(fileName, 1));
            }
            CreateFileWithSize(fileName, maximumFileSize * 2);
            string expected = AppendFileCounter(fileName, 1);
            string actual;
            actual = target.GetRolledFileName(fileName);
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for GetRolledFileName - max size > 0 rollover - file name, path and extension.
        ///</summary>
        [TestMethod()]
        public void GetRolledFileNameTest7()
        {
            int maximumFileSize = 1;
            FileSizeRolloverStrategy target = new FileSizeRolloverStrategy(maximumFileSize);
            string fileName = System.IO.Path.Combine(System.IO.Path.GetTempPath(), "GetRolledFileNameTest7.txt");
            if (System.IO.File.Exists(fileName))
            {
                System.IO.File.Delete(fileName);
            }
            if (System.IO.File.Exists(AppendFileCounter(fileName, 1)))
            {
                System.IO.File.Delete(AppendFileCounter(fileName, 1));
            }
            CreateFileWithSize(fileName, maximumFileSize * 2);
            string expected = AppendFileCounter(fileName, 1);
            string actual;
            actual = target.GetRolledFileName(fileName);
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for GetRolledFileName - max size > 0 rollover - file name, path and no extension.
        ///</summary>
        [TestMethod()]
        public void GetRolledFileNameTest8()
        {
            int maximumFileSize = 1;
            FileSizeRolloverStrategy target = new FileSizeRolloverStrategy(maximumFileSize);
            string fileName = System.IO.Path.Combine(System.IO.Path.GetTempPath(), "GetRolledFileNameTest8.txt");
            if (System.IO.File.Exists(fileName))
            {
                System.IO.File.Delete(fileName);
            }
            if (System.IO.File.Exists(AppendFileCounter(fileName, 1)))
            {
                System.IO.File.Delete(AppendFileCounter(fileName, 1));
            }
            CreateFileWithSize(fileName, maximumFileSize * 2);
            string expected = AppendFileCounter(fileName, 1);
            string actual;
            actual = target.GetRolledFileName(fileName);
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for GetRolledFileName - max size > 0 rollover - file name, path and no extension.
        ///</summary>
        [TestMethod()]
        public void GetRolledFileNameTest9()
        {
            int maximumFileSize = 1;
            FileSizeRolloverStrategy target = new FileSizeRolloverStrategy(maximumFileSize);
            string fileName = System.IO.Path.Combine(System.IO.Path.GetTempPath(), "GetRolledFileNameTest9.txt");
            if (System.IO.File.Exists(fileName))
            {
                System.IO.File.Delete(fileName);
            }
            if (System.IO.File.Exists(AppendFileCounter(fileName, 1)))
            {
                System.IO.File.Delete(AppendFileCounter(fileName, 1));
            }
            if (System.IO.File.Exists(AppendFileCounter(fileName, 2)))
            {
                System.IO.File.Delete(AppendFileCounter(fileName, 2));
            }
            CreateFileWithSize(fileName, maximumFileSize * 2);
            string expected = AppendFileCounter(fileName, 1);
            string actual;
            actual = target.GetRolledFileName(fileName);
            Assert.AreEqual(expected, actual, "First rollover");
            CreateFileWithSize(actual, maximumFileSize * 2);
            expected = AppendFileCounter(fileName,2);
            actual = target.GetRolledFileName(fileName);
            Assert.AreEqual(expected, actual, "Second rollover");
        }
        #endregion

        #region Private Methods

        private void CreateFileWithSize(string fileName, int size)
        {
            string contents = "";
            while (contents.Length <= size * 1024)
            {
                contents = contents + "                                                           ";
            }
            contents = contents.Substring(0, size * 1024);
            System.IO.File.WriteAllText(fileName, contents);
        }

        /// <summary>
        /// Append the counter value to the supplied file name.
        /// </summary>
        /// <param name="fileName"></param>
        /// <param name="counter"></param>
        /// <returns></returns>
        /// <remarks>
        /// Contract: pre  : fileName must not be null.
        ///           post : Result will not be null.
        /// </remarks>
        private string AppendFileCounter(string fileName, int counter)
        {
            Contract.Requires(fileName != null);
            Contract.Ensures(Contract.Result<string>() != null);

            string ext = System.IO.Path.GetExtension(fileName);
            int len = fileName.Length - ext.Length;
            if (0 < len) // len should always be greater than 1, here to satisfy contract for string.Substring.
            {
                return fileName.Substring(0, len) + counter.ToString() + ext;
            }
            else
            {
                return counter.ToString() + ext;
            }
        }

        #endregion


        #region Contract Tests

        /// <summary>
        /// Test contract volilation when creating instance with negative max file size value.
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void NegativeMaxSizeContractTest1()
        {
            try
            {
                int maximumFileSize = -1;
                FileSizeRolloverStrategy target = new FileSizeRolloverStrategy(maximumFileSize);
            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: NegativeMaxSizeContractTest1", ex);
                }
                else
                {
                    throw;
                }
            }
        }


        #endregion
    }
}
