﻿using tpts.Logging;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Diagnostics.Contracts;

namespace tptsLoggingUnitTests
{


    /// <summary>
    ///This is a test class for LogEntryTest and is intended
    ///to contain all LogEntryTest Unit Tests
    ///</summary>
    [TestClass()]
    public class LogEntryTest
    {


        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion

        /// <summary>
        /// Constructor for LogEntryTest
        /// </summary>
        public LogEntryTest()
        {
            // Add event handler for contract failures, required so tests can trap any contract exceptions
            // Required to allow unit tests to test contracts are correct.
            Contract.ContractFailed += (sender, e) =>
            {
                e.SetHandled();
                e.SetUnwind();
                throw new ContractTestFailedException(e.FailureKind + ": " + e.Message);
            };
        }


        #region Constructor Tests

 

        /// <summary>
        ///A test for LogEntry`1 Constructor, using string entry, severity and category.
        ///</summary>
        public void LogEntryConstructorTest1Helper()
        {
            string entry = "Test Entry Text";
            LogEntrySeverity severity = LogEntrySeverity.Critical;
            string category = "Test Category";
            DateTime entryDateTime = DateTime.Now;
            LogEntry<string> target = new LogEntry<string>(entryDateTime,entry, severity, category);
            Assert.AreEqual(entry, target.Entry);
            Assert.AreEqual(severity, target.Severity);
            Assert.AreEqual(category, target.Category);
            Assert.AreEqual(entryDateTime, target.LogEntryDateTime);
        }

        [TestMethod()]
        public void LogEntryConstructorTest1()
        {
            LogEntryConstructorTest1Helper();
        }

        /// <summary>
        ///A test for LogEntry`1 Constructor, using LogEntryType entry and category.
        ///</summary>
        public void LogEntryConstructorTest6Helper()
        {
            LogEntryType entry = new LogEntryType();
            int expectedInt = 5;
            string expectedString = "Test Log Entry Type";
            entry.IntegerProperty = expectedInt;
            entry.StringProperty = expectedString;

            LogEntrySeverity severity = LogEntrySeverity.Debug;
            string category = "Dummy Category";
            DateTime entryDateTime = DateTime.Now;
            LogEntry<LogEntryType> target = new LogEntry<LogEntryType>(entryDateTime,entry, severity, category);
            Assert.AreEqual(entry, target.Entry);
            Assert.AreEqual(expectedString, target.Entry.StringProperty);
            Assert.AreEqual(expectedInt, target.Entry.IntegerProperty);
            Assert.AreEqual(severity, target.Severity);
            Assert.AreEqual(category, target.Category);
            Assert.AreEqual(entryDateTime, target.LogEntryDateTime);
        }

        [TestMethod()]
        public void LogEntryConstructorTest6()
        {
            LogEntryConstructorTest6Helper();
        }

        #endregion

        #region Property Tests


        #endregion

        #region Contract Tests

        /// <summary>
        ///A test for null category passed through to constructor fails contract.
        ///</summary>
        public void LogEntryConstructorTest4Helper()
        {
            string entry = "Test Entry Text";
            LogEntrySeverity severity = LogEntrySeverity.Information;
            string category = null;
            DateTime entryDateTime = DateTime.Now;
            LogEntry<string> target = new LogEntry<string>(entryDateTime,entry, severity, category);
            Assert.AreEqual(entry, target.Entry);
            Assert.AreEqual(severity, target.Severity);
            Assert.AreEqual(category, target.Category);
            Assert.AreEqual(entryDateTime, target.LogEntryDateTime);
        }

        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogEntryConstructorTest4()
        {
            try
            {
                LogEntryConstructorTest4Helper();
            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogEntryConstructorTest4", ex);
                }
                else
                {
                    throw;
                }
            }

        }

        /// <summary>
        /// <summary>
        ///A test for null string entry passed through to constructor fails contract.
        ///</summary>
        ///</summary>
        public void LogEntryConstructorTest5Helper()
        {
            string entry = null;
            LogEntrySeverity severity = LogEntrySeverity.Information;
            string category = "";
            DateTime entryDateTime = DateTime.Now;
            LogEntry<string> target = new LogEntry<string>(entryDateTime, entry, severity, category);
            Assert.AreEqual(entry, target.Entry);
            Assert.AreEqual(severity, target.Severity);
            Assert.AreEqual(category, target.Category);
            Assert.AreEqual(entryDateTime, target.LogEntryDateTime);
        }

        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogEntryConstructorTest5()
        {
            try
            {
                LogEntryConstructorTest5Helper();
            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogEntryConstructorTest5", ex);
                }
                else
                {
                    throw;
                }
            }

        }

        #endregion
    }
}
