﻿using tpts.Logging;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Diagnostics.Contracts;
using System.Collections.Generic;

namespace tptsLoggingUnitTests
{


    /// <summary>
    ///This is a test class for MemoryLoggerTest and is intended
    ///to contain all MemoryLoggerTest Unit Tests
    ///</summary>
    [TestClass()]
    public class MemoryLoggerTest
    {


        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion

        /// <summary>
        /// Constructor for ConsoleLoggerTest
        /// </summary>
        public MemoryLoggerTest()
        {
            // Add event handler for contract failures, required so tests can trap any contract exceptions
            // Required to allow unit tests to test contracts are correct.
            Contract.ContractFailed += (sender, e) =>
            {
                e.SetHandled();
                e.SetUnwind();
                throw new ContractTestFailedException(e.FailureKind + ": " + e.Message);
            };
        }


        #region Constructor Tests

        /// <summary>
        ///A test for MemoryLogger`1 Constructor
        ///</summary>
        public void MemoryLoggerConstructorTestHelper<T>()
        {
            MemoryLogger<T> target = new MemoryLogger<T>();
            Assert.AreEqual(0, target.Count);
        }

        [TestMethod()]
        public void MemoryLoggerConstructorTest()
        {
            MemoryLoggerConstructorTestHelper<GenericParameterHelper>();
        }

        #endregion

        #region Methods Tests

        /// <summary>
        ///A test for Clear
        ///</summary>
        [TestMethod()]
        public void ClearTest()
        {
            MemoryLogger<string> target = new MemoryLogger<string>();
            Random r = new Random();
            int itemCount = r.Next(10, 100);
            for (int index = 1; index <= itemCount; index++)
            {
                target.Log("ClearTestHelper Entry");
            }
            Assert.AreEqual(itemCount, target.Count);
            target.Clear();
            Assert.AreEqual(0, target.Count);
        }

        /// <summary>
        ///A test for Contains
        ///</summary>
        [TestMethod()]
        public void ContainsTest()
        {
            List<string> items = new List<string>();
            Random r = new Random();
            MemoryLogger<string> target = new MemoryLogger<string>();
            int itemCount = r.Next(10, 100);
            for (int index = 1; index <= itemCount; index++)
            {
                target.Log("ContainsTest Entry");
            }
            Assert.AreEqual(itemCount, target.Count);

            for (int index = 0; index < itemCount; index++)
            {
                ReadOnlyLogEntry entry = target.ItemAt(index);
             //   ReadOnlyLogEntry entry = target.ItemAt(index);
                Assert.IsTrue(target.Contains(entry));
            }
        }

        /// <summary>
        ///A test for GetCurrentEntries
        ///</summary>
        public void GetCurrentEntriesTestHelper<T>()
        {
        }

        [TestMethod()]
        public void GetCurrentEntriesTest()
        {
            MemoryLogger<string> target = new MemoryLogger<string>();
            Random r = new Random();
            int itemCount = r.Next(10, 100);
            for (int index = 1; index <= itemCount; index++)
            {
                target.Log("ContainsTest Entry");
            }
            Assert.AreEqual(itemCount, target.Count);
            ICollection<ReadOnlyLogEntry> actual;
            actual = target.GetCurrentEntries();
            Assert.AreEqual(itemCount, actual.Count);
            for (int index = 0; index < itemCount; index++)
            {
                ReadOnlyLogEntry entry = target.ItemAt(index);
                Assert.IsTrue(actual.Contains(entry));
            }
        }

        /// <summary>
        ///A test for GetEnumerator
        ///</summary>
        [TestMethod()]
        public void GetEnumeratorTest()
        {
            MemoryLogger<string> target = new MemoryLogger<string>();
            Random r = new Random();
            int itemCount = r.Next(10, 100);
            for (int index = 1; index <= itemCount; index++)
            {
                target.Log("GetEnumeratorTestHelper Entry");
            }
            Assert.AreEqual(itemCount, target.Count);
            ICollection<ReadOnlyLogEntry> actual;
            actual = target.GetCurrentEntries();
            Assert.AreEqual(itemCount, actual.Count);
            foreach (ReadOnlyLogEntry entry in target)
            {
                Assert.IsTrue(target.Contains(entry));
            }
            foreach (ReadOnlyLogEntry entry in target)
            {
                Assert.IsTrue(target.Contains(entry));
                target.Log("GetEnumeratorTestHelper During Enumeration");
            }
        }

        /// <summary>
        ///A test for Remove
        ///</summary>
        [TestMethod()]
        public void RemoveTest()
        {
            MemoryLogger<string> target = new MemoryLogger<string>();
            Random r = new Random();
            int itemCount = r.Next(10, 100);
            for (int index = 1; index <= itemCount; index++)
            {
                target.Log("RemoveTestHelper Entry");
            }
            Assert.AreEqual(itemCount, target.Count);
            foreach (ReadOnlyLogEntry entry in target)
            {
                Assert.IsTrue(target.Contains(entry));
                Assert.AreEqual(itemCount, target.Count);
                target.Remove(entry);
                itemCount -= 1;
                Assert.IsFalse(target.Contains(entry));
                Assert.AreEqual(itemCount, target.Count);
            }
        }

        /// <summary>
        ///A test for Count
        ///</summary>
        [TestMethod()]
        public void CountTest()
        {
            MemoryLogger<string> target = new MemoryLogger<string>();
            Random r = new Random();
            int itemCount = r.Next(10, 100);
            for (int index = 1; index <= itemCount; index++)
            {
                target.Log("CountTest Entry");
            }
            Assert.AreEqual(itemCount, target.Count);
        }

        #endregion

        #region Standard Logger Tests

        [TestMethod()]
        public void StandardLoggerTests()
        {
            ILoggerTestHelper loggerTests = new ILoggerTestHelper();
            loggerTests.RunStandardTests(CreateMemoryLogger<string>());
        }

        #endregion

        #region Contract Tests

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest1()
        {
            try
            {
                MemoryLogger<string> logger = CreateMemoryLogger<string>();
                string entryText = null;

                logger.Log(entryText);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest1", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest2()
        {
            try
            {
                MemoryLogger<string> logger = CreateMemoryLogger<string>();
                string entryText = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;

                logger.Log(entryText, severity);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest2", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest3()
        {
            try
            {
                MemoryLogger<string> logger = CreateMemoryLogger<string>();
                string entryText = null;
                string category = "LogNullEntryContractTest3 Category";

                logger.Log(entryText, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest3", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest4()
        {
            try
            {
                MemoryLogger<string> logger = CreateMemoryLogger<string>();
                string entryText = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;
                string category = "LogNullEntryContractTest4 Category";

                logger.Log(entryText, severity, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest4", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null category
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullCategoryContractTest1()
        {
            try
            {
                MemoryLogger<string> logger = CreateMemoryLogger<string>();
                string entryText = "LogNullCategoryContractTest1 Entry";
                string category = null;

                logger.Log(entryText, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullCategoryContractTest1", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullCategoryContractTest2()
        {
            try
            {
                MemoryLogger<string> logger = CreateMemoryLogger<string>();
                string entryText = "LogNullCategoryContractTest2 Entry";
                string category = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;

                logger.Log(entryText, severity, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullCategoryContractTest2", ex);
                }
                else
                {
                    throw;
                }
            }
        }
        #endregion

        #region Private Methods

        private MemoryLogger<T> CreateMemoryLogger<T>()
        {
            MemoryLogger<T> target = new MemoryLogger<T>();
            return target;
        }

        #endregion

    }
}
