﻿using tpts.Logging;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Diagnostics.Contracts;

namespace tptsLoggingUnitTests
{


    /// <summary>
    ///This is a test class for RollingFileLoggerTest and is intended
    ///to contain all RollingFileLoggerTest Unit Tests
    ///</summary>
    [TestClass()]
    public class RollingFileLoggerTest
    {


        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion

        /// <summary>
        /// Constructor for RollingFileLoggerTest
        /// </summary>
        public RollingFileLoggerTest()
        {
            // Add event handler for contract failures, required so tests can trap any contract exceptions
            // Required to allow unit tests to test contracts are correct.
            Contract.ContractFailed += (sender, e) =>
            {
                e.SetHandled();
                e.SetUnwind();
                throw new ContractTestFailedException(e.FailureKind + ": " + e.Message);
            };
        }

        #region LogFileName Test

        /// <summary>
        ///A test for LogFileName - file name and extension.
        ///</summary>
        [TestMethod()]
        public void LogFileNameTest1()
        {
            string logFileName = "RollingLoggerTestLogFileNameTest1.txt";
            FileLogger<string> fileLogger = new FileLogger<string>(logFileName);
            FixedDateRolloverStrategy rolloverStrategy = new FixedDateRolloverStrategy();
            RollingFileLogger<string> rollingLogger = new RollingFileLogger<string>(fileLogger, rolloverStrategy);
            string expected = rolloverStrategy.GetRolledFileName(logFileName);
            string actual;
            actual = rollingLogger.LogFileName;
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for LogFileName - file name and no extension.
        ///</summary>
        [TestMethod()]
        public void LogFileNameTest2()
        {
            string logFileName = "RollingLoggerTestLogFileNameTest2";
            FileLogger<string> fileLogger = new FileLogger<string>(logFileName);
            FixedDateRolloverStrategy rolloverStrategy = new FixedDateRolloverStrategy();
            RollingFileLogger<string> rollingLogger = new RollingFileLogger<string>(fileLogger, rolloverStrategy);
            string expected = rolloverStrategy.GetRolledFileName(logFileName);
            string actual;
            actual = rollingLogger.LogFileName;
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for LogFileName - file name, path and  extension.
        ///</summary>
        [TestMethod()]
        public void LogFileNameTest3()
        {
            string logFileName = System.IO.Path.Combine(System.IO.Path.GetTempPath(), "LogFileNameTest3.txt");
            FileLogger<string> fileLogger = new FileLogger<string>(logFileName);
            FixedDateRolloverStrategy rolloverStrategy = new FixedDateRolloverStrategy();
            RollingFileLogger<string> rollingLogger = new RollingFileLogger<string>(fileLogger, rolloverStrategy);
            string expected = rolloverStrategy.GetRolledFileName(logFileName);
            string actual;
            actual = rollingLogger.LogFileName;
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for LogFileName - file name, path and no  extension.
        ///</summary>
        [TestMethod()]
        public void LogFileNameTest4()
        {
            string logFileName = System.IO.Path.Combine(System.IO.Path.GetTempPath(), "LogFileNameTest4");
            FileLogger<string> fileLogger = new FileLogger<string>(logFileName);
            FixedDateRolloverStrategy rolloverStrategy = new FixedDateRolloverStrategy();
            RollingFileLogger<string> rollingLogger = new RollingFileLogger<string>(fileLogger, rolloverStrategy);
            string expected = rolloverStrategy.GetRolledFileName(logFileName);
            string actual;
            actual = rollingLogger.LogFileName;
            Assert.AreEqual(expected, actual);
        }

        #endregion


        #region Logging Test

        /// <summary>
        ///A test for LogFileName - file name and extension.
        ///</summary>
        [TestMethod()]
        public void LogTest1()
        {
            string logFileName = "RollingLoggerTestLogTest1.txt";
            FileLogger<string> fileLogger = new FileLogger<string>(logFileName);
            FixedDateRolloverStrategy rolloverStrategy = new FixedDateRolloverStrategy();
            TestLog(logFileName, fileLogger, rolloverStrategy);
        }


        /// <summary>
        ///A test for LogFileName - file name and no extension.
        ///</summary>
        [TestMethod()]
        public void LogTest2()
        {
            string logFileName = "RollingLoggerTestLogTest2";
            FileLogger<string> fileLogger = new FileLogger<string>(logFileName);
            FixedDateRolloverStrategy rolloverStrategy = new FixedDateRolloverStrategy();
            TestLog(logFileName, fileLogger, rolloverStrategy);
        }

        /// <summary>
        ///A test for LogFileName - file name, path and  extension.
        ///</summary>
        [TestMethod()]
        public void LogTest3()
        {
            string logFileName = System.IO.Path.Combine(System.IO.Path.GetTempPath(), "LogTest3.txt");
            FileLogger<string> fileLogger = new FileLogger<string>(logFileName);
            FixedDateRolloverStrategy rolloverStrategy = new FixedDateRolloverStrategy();
            TestLog(logFileName, fileLogger, rolloverStrategy);
        }

        /// <summary>
        ///A test for LogFileName - file name, path and no  extension.
        ///</summary>
        [TestMethod()]
        public void LogTest4()
        {
            string logFileName = System.IO.Path.Combine(System.IO.Path.GetTempPath(), "LogTest4");
            FileLogger<string> fileLogger = new FileLogger<string>(logFileName);
            FixedDateRolloverStrategy rolloverStrategy = new FixedDateRolloverStrategy();
            TestLog(logFileName, fileLogger, rolloverStrategy);
        }

        #endregion

        #region Standard Logger Tests

        [TestMethod()]
        public void StandardLoggerTests()
        {
            ILoggerTestHelper loggerTests = new ILoggerTestHelper();
            string logFileName = "RollingLoggerTestStandardLoggerTests.txt";
            loggerTests.RunStandardTests(CreateRollingFileLogger<string>(logFileName));
        }


        #endregion

        #region Contract Tests

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest1()
        {
            try
            {
                string logFileName = "RollingLoggerTestLogNullEntryContractTest1.txt";
                RollingFileLogger<string> logger = CreateRollingFileLogger<string>(logFileName);
                string entryText = null;

                logger.Log(entryText);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest1", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest2()
        {
            try
            {
                string logFileName = "RollingLoggerTestLogNullEntryContractTest2.txt";
                RollingFileLogger<string> logger = CreateRollingFileLogger<string>(logFileName);
                string entryText = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;

                logger.Log(entryText, severity);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest2", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest3()
        {
            try
            {
                string logFileName = "RollingLoggerTestLogNullEntryContractTest3.txt";
                RollingFileLogger<string> logger = CreateRollingFileLogger<string>(logFileName);
                string entryText = null;
                string category = "LogNullEntryContractTest3 Category";

                logger.Log(entryText, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest3", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest4()
        {
            try
            {
                string logFileName = "RollingLoggerTestLogNullEntryContractTest4.txt";
                RollingFileLogger<string> logger = CreateRollingFileLogger<string>(logFileName);
                string entryText = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;
                string category = "LogNullEntryContractTest4 Category";

                logger.Log(entryText, severity, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest4", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null category
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullCategoryContractTest1()
        {
            try
            {
                string logFileName = "RollingLoggerTestLogNullCategoryContractTest1.txt";
                RollingFileLogger<string> logger = CreateRollingFileLogger<string>(logFileName);
                string entryText = "LogNullCategoryContractTest1 Entry";
                string category = null;

                logger.Log(entryText, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullCategoryContractTest1", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullCategoryContractTest2()
        {
            try
            {
                string logFileName = "RollingLoggerTestLogNullCategoryContractTest2.txt";
                RollingFileLogger<string> logger = CreateRollingFileLogger<string>(logFileName);
                string entryText = "LogNullCategoryContractTest2 Entry";
                string category = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;

                logger.Log(entryText, severity, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullCategoryContractTest2", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when creating instance with null file logger.
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void ConstructorNullContractTest1()
        {
            try
            {
                //string logFileName = "RollingLoggerTestConstructorNullContractTest1.txt";
                FileLogger<string> fileLogger = null; // new FileLogger<string>(logFileName);
                FixedDateRolloverStrategy rolloverStrategy = new FixedDateRolloverStrategy();
                RollingFileLogger<string> rollingLogger = new RollingFileLogger<string>(fileLogger, rolloverStrategy);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: ConstructorNullContractTest1", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when creating instance with null rollover strategy.
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void ConstructorNullContractTest2()
        {
            try
            {
                string logFileName = "RollingLoggerTestConstructorNullContractTest2.txt";
                FileLogger<string> fileLogger = new FileLogger<string>(logFileName);
                FixedDateRolloverStrategy rolloverStrategy = null;//new FixedDateRolloverStrategy();
                RollingFileLogger<string> rollingLogger = new RollingFileLogger<string>(fileLogger, rolloverStrategy);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: ConstructorNullContractTest2", ex);
                }
                else
                {
                    throw;
                }
            }
        }
        #endregion

        #region Private Methods

        private RollingFileLogger<T> CreateRollingFileLogger<T>(string logFileName)
        {
            FileLogger<T> fileLogger = new FileLogger<T>(logFileName);
            FixedDateRolloverStrategy rolloverStrategy = new FixedDateRolloverStrategy();
            return new RollingFileLogger<T>(fileLogger, rolloverStrategy);
        }

        private void TestLog(string logFileName, FileLogger<string> fileLogger, IFileRolloverStrategy rolloverStrategy)
        {
            RollingFileLogger<string> rollingLogger = new RollingFileLogger<string>(fileLogger, rolloverStrategy);
            string expected = rolloverStrategy.GetRolledFileName(logFileName);
            string actual;
            actual = rollingLogger.LogFileName;
            Assert.AreEqual(expected, actual);
            if (System.IO.File.Exists(actual))
            {
                System.IO.File.Delete(actual);
            }
            Assert.AreEqual(false, System.IO.File.Exists(actual));
            rollingLogger.Log("LogTest1 Entry");
            Assert.AreEqual(true, System.IO.File.Exists(actual));
        }

        #endregion

    }
}
