﻿using tpts.Logging;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Diagnostics.Contracts;

namespace tptsLoggingUnitTests
{


    /// <summary>
    ///This is a test class for RollingXMLFileLoggerTest and is intended
    ///to contain all RollingXMLFileLoggerTest Unit Tests
    ///</summary>
    [TestClass()]
    public class RollingXMLFileLoggerTest
    {

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion

        /// <summary>
        /// Constructor for RollingXMLFileLoggerTest
        /// </summary>
        public RollingXMLFileLoggerTest()
        {
            // Add event handler for contract failures, required so tests can trap any contract exceptions
            // Required to allow unit tests to test contracts are correct.
            Contract.ContractFailed += (sender, e) =>
            {
                e.SetHandled();
                e.SetUnwind();
                throw new ContractTestFailedException(e.FailureKind + ": " + e.Message);
            };
        }

        #region Logging Test

        /// <summary>
        ///A test for LogFileName - file name and extension.
        ///</summary>
        [TestMethod()]
        public void LogTest1()
        {
            string logFileName = "RollingXMLLoggerTestLogTest1.xml";
            IFileRolloverStrategy strategy;
            FileLogger<string> logger = CreateRollingFileLogger<string>(logFileName, out strategy);
            TestLogMethod(logFileName, logger, strategy);
        }

        /// <summary>
        ///A test for LogFileName - file name and no extension.
        ///</summary>
        [TestMethod()]
        public void LogTest2()
        {
            string logFileName = "RollingXMLLoggerTestLogTest2";
            IFileRolloverStrategy strategy;
            FileLogger<string> logger = CreateRollingFileLogger<string>(logFileName, out strategy);
            TestLogMethod(logFileName, logger, strategy);
        }

        /// <summary>
        ///A test for LogFileName - file name, path and  extension.
        ///</summary>
        [TestMethod()]
        public void LogTest3()
        {
            string logFileName = System.IO.Path.Combine(System.IO.Path.GetTempPath(), "LogTest3.xml");
            IFileRolloverStrategy strategy;
            FileLogger<string> logger = CreateRollingFileLogger<string>(logFileName, out strategy);
            TestLogMethod(logFileName, logger, strategy);
        }

        /// <summary>
        ///A test for LogFileName - file name, path and no  extension.
        ///</summary>
        [TestMethod()]
        public void LogTest4()
        {
            string logFileName = System.IO.Path.Combine(System.IO.Path.GetTempPath(), "LogTest4");
            IFileRolloverStrategy strategy;
            FileLogger<string> logger = CreateRollingFileLogger<string>(logFileName, out strategy);
            TestLogMethod(logFileName, logger, strategy);
        }

        #endregion


        #region Standard Logger Tests

        [TestMethod()]
        public void StandardLoggerTests()
        {
            ILoggerTestHelper loggerTests = new ILoggerTestHelper();
            string logFileName = "RollingXMLLoggerTestStandardLoggerTests.xml";
            IFileRolloverStrategy strategy;
            FileLogger<string> logger = CreateRollingFileLogger<string>(logFileName, out strategy);
            loggerTests.RunStandardTests(logger);
        }


        #endregion

        #region Contract Tests

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest1()
        {
            try
            {
                string logFileName = "RollingXMLLoggerTestLogNullEntryContractTest1.xml";
                IFileRolloverStrategy strategy;
                FileLogger<string> logger = CreateRollingFileLogger<string>(logFileName, out strategy);
                string entryText = null;

                logger.Log(entryText);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest1", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest2()
        {
            try
            {
                string logFileName = "RollingXMLLoggerTestLogNullEntryContractTest2.xml";
                IFileRolloverStrategy strategy;
                FileLogger<string> logger = CreateRollingFileLogger<string>(logFileName, out strategy);
                string entryText = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;

                logger.Log(entryText, severity);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest2", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest3()
        {
            try
            {
                string logFileName = "RollingXMLLoggerTestLogNullEntryContractTest3.xml";
                IFileRolloverStrategy strategy;
                FileLogger<string> logger = CreateRollingFileLogger<string>(logFileName, out strategy);
                string entryText = null;
                string category = "LogNullEntryContractTest3 Category";

                logger.Log(entryText, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest3", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest4()
        {
            try
            {
                string logFileName = "RollingXMLLoggerTestLogNullEntryContractTest4.xml";
                IFileRolloverStrategy strategy;
                FileLogger<string> logger = CreateRollingFileLogger<string>(logFileName, out strategy);
                string entryText = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;
                string category = "LogNullEntryContractTest4 Category";

                logger.Log(entryText, severity, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest4", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null category
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullCategoryContractTest1()
        {
            try
            {
                string logFileName = "RollingXMLLoggerTestLogNullCategoryContractTest1.xml";
                IFileRolloverStrategy strategy;
                FileLogger<string> logger = CreateRollingFileLogger<string>(logFileName, out strategy);
                string entryText = "LogNullCategoryContractTest1 Entry";
                string category = null;

                logger.Log(entryText, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullCategoryContractTest1", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullCategoryContractTest2()
        {
            try
            {
                string logFileName = "RollingXMLLoggerTestLogNullCategoryContractTest2.xml";
                IFileRolloverStrategy strategy;
                FileLogger<string> logger = CreateRollingFileLogger<string>(logFileName, out strategy);
                string entryText = "LogNullCategoryContractTest2 Entry";
                string category = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;

                logger.Log(entryText, severity, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullCategoryContractTest2", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        #endregion

        #region Private Methods

        private XMLFileLogger<T> CreateXMLFileLogger<T>(string logFileName)
        {
            FileLogger<T> fileLogger = new FileLogger<T>(logFileName);
            return new XMLFileLogger<T>(fileLogger);
        }

        private RollingFileLogger<T> CreateRollingFileLogger<T>(string logFileName, out IFileRolloverStrategy strategy)
        {
            XMLFileLogger<T> xmlLogger = CreateXMLFileLogger<T>(logFileName);
            strategy = new FixedDateRolloverStrategy();
            return new RollingFileLogger<T>(xmlLogger, strategy);
        }

        private string GetExpectedDataFileName(string logFileName)
        {
            string ext = System.IO.Path.GetExtension(logFileName);
            int len = logFileName.Length - ext.Length;
            if (0 < len) // len should always be greater than 1, here to satisfy contract for string.Substring.
            {
                return logFileName.Substring(0, len) + "_Entries" + ext;
            }
            else
            {
                return "_Entries" + ext;
            }
        }

        private void TestLogMethod(string logFileName, FileLogger<string> logger, IFileRolloverStrategy rolloverStrategy)
        {
            FixedFormatter<string> formatter = new FixedFormatter<string>();
            logger.LogEntryFormatter = formatter;
            string expected = rolloverStrategy.GetRolledFileName(logFileName);
            string actual;
            actual = logger.LogFileName;
            Assert.AreEqual(expected, actual);

            Console.WriteLine(System.IO.Path.GetTempPath());

            expected = GetExpectedDataFileName(expected);

            if (System.IO.File.Exists(actual))
            {
                System.IO.File.Delete(actual);
            }
            if (System.IO.File.Exists(expected))
            {
                System.IO.File.Delete(expected);
            }
            Assert.AreEqual(false, System.IO.File.Exists(actual));
            Assert.AreEqual(false, System.IO.File.Exists(expected));
            logger.Log("LogTest1 Entry");
            Assert.AreEqual(true, System.IO.File.Exists(actual), "Header file does not exist " + actual);
            Assert.AreEqual(true, System.IO.File.Exists(expected), "Data file does not exist " + expected);
            if (System.IO.File.Exists(actual))
            {
                string contents = System.IO.File.ReadAllText(actual);
                Assert.IsTrue(contents.Contains("xml"));
            }
            if (System.IO.File.Exists(actual))
            {
                string contents = System.IO.File.ReadAllText(actual);
                Assert.IsTrue(contents.Contains("xml"));
                Assert.IsFalse(contents.Contains("LogTest1 Entry"));
            }
            if (System.IO.File.Exists(expected))
            {
                string contents = System.IO.File.ReadAllText(expected);
                Assert.IsTrue(contents.Contains("LogTest1 Entry"));
            }

        }
        #endregion


    }
}
