﻿using tpts.Logging;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Diagnostics.Contracts;

namespace tptsLoggingUnitTests
{


    /// <summary>
    ///This is a test class for StandardLogEntryFormatterTest and is intended
    ///to contain all StandardLogEntryFormatterTest Unit Tests
    ///</summary>
    [TestClass()]
    public class StandardLogEntryFormatterTest
    {


        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion

        /// <summary>
        /// Constructor for StandardLogEntryFormatterTest
        /// </summary>
        public StandardLogEntryFormatterTest()
        {
            // Add event handler for contract failures, required so tests can trap any contract exceptions
            // Required to allow unit tests to test contracts are correct.
            Contract.ContractFailed += (sender, e) =>
            {
                e.SetHandled();
                e.SetUnwind();
                throw new ContractTestFailedException(e.FailureKind + ": " + e.Message);
            };
        }


        #region Constructor Tests

        /// <summary>
        ///A test for StandardLogEntryFormatter`1 Constructor
        ///</summary>
        [TestMethod()]
        public void StandardLogEntryFormatterConstructorTest1()
        {
            string dateTimeFormat = "yyyy-MM-dd hh:mm:ss";
            StandardLogEntryFormatter<string> formatter = new StandardLogEntryFormatter<string>(dateTimeFormat);
            string actual;
            actual = formatter.DateTimeFormat;
            Assert.AreEqual(dateTimeFormat, actual);
        }

        /// <summary>
        ///A test for StandardLogEntryFormatter`2 Constructor
        ///</summary>
        [TestMethod()]
        public void StandardLogEntryFormatterConstructorTest2()
        {
            string dateTimeFormat = "yyyy-MMM-dd hh:mm:ss";
            StandardLogEntryFormatter<string> formatter = new StandardLogEntryFormatter<string>(dateTimeFormat);
            string actual;
            actual = formatter.DateTimeFormat;
            Assert.AreEqual(dateTimeFormat, actual);
        }

        #endregion


         [TestMethod()]
        public void FormatEntryTest()
        {
            string dateTimeFormat = "yyyy-MM-dd hh:mm:ss";
            StandardLogEntryFormatter<string> formatter = CreateStandardLogEntryFormatter(dateTimeFormat);
            FixedLogEntry<string> fixedLogEntry = new FixedLogEntry<string>();
            string actual;
            actual = formatter.FormatEntry(fixedLogEntry);
            Assert.IsTrue(actual.Contains(fixedLogEntry.Entry), "Does Not Contain Entry Text");
            Assert.IsTrue(actual.Contains(fixedLogEntry.Category), "Does Not Contain Category");
            Assert.IsTrue(actual.Contains(fixedLogEntry.Severity.ToString()), "Does Not Contain Severity");
            Assert.IsTrue(actual.Contains(fixedLogEntry.LogEntryDateTime.ToString(dateTimeFormat)), "Does Not Contain Entry Date/Time");
        }

 
        #region Contract Tests

        /// <summary>
        /// Test contract volilation when creating passing null date time format parameter to constructor.
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void ConstructorNullEntryContractTest1()
        {
            try
            {
                string dateTimeFormat = null;
                CreateStandardLogEntryFormatter(dateTimeFormat);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: ConstructorNullEntryContractTest1", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when creating passing blank date time format parameter to constructor.
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void ConstructorNullEntryContractTest2()
        {
            try
            {
                string dateTimeFormat = "";
                CreateStandardLogEntryFormatter(dateTimeFormat);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: ConstructorNullEntryContractTest2", ex);
                }
                else
                {
                    throw;
                }
            }
        }

 
        #endregion

        #region Private Methods

        private StandardLogEntryFormatter<string> CreateStandardLogEntryFormatter(string dateTimeFormat)
        {
            return new StandardLogEntryFormatter<string>(dateTimeFormat);
        }

#endregion
    }
}
