﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using tpts.Logging;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Diagnostics.Contracts;

namespace tptsLoggingUnitTests
{
    class ILoggerTestHelper
    {

 

        /// <summary>
        /// Run the standard logger tests to ensure that the standard contracts and basic logging functionaility is correct.
        /// </summary>
        /// <param name="logger"></param>
        public void RunStandardTests(ILogger<string> logger)
        {
            LogTest1(logger);
            LogTest2(logger);
            LogTest3(logger);
            LogTest4(logger);
            LogTest5(logger);
            LogTest6(logger);
            LogTest7(logger);
            LogTest8(logger);
            LogTest9(logger);
            LogTest10(logger);
        }

        /// <summary>
        /// Test Log entry, no filter.
        /// </summary>
        private void LogTest1(ILogger<string> logger)
        {
            logger.LogEntryFilter = null;

            string entryText = "LogTest1";
            bool expected = true;
            bool actual = logger.Log(entryText);
            Assert.AreEqual(expected, actual, "LogTest1");
        }


        /// <summary>
        /// Test Log entry, severity, no filter.
        /// </summary>

        private void LogTest2(ILogger<string> logger)
        {

            logger.LogEntryFilter = null;

            string entryText = "LogTest2";
            LogEntrySeverity severity = LogEntrySeverity.Information;
            bool expected = true;
            bool actual = logger.Log(entryText, severity);
            Assert.AreEqual(expected, actual, "LogTest2");
        }

        /// <summary>
        /// Test Log entry, category, no filter.
        /// </summary>

        private void LogTest3(ILogger<string> logger)
        {

            logger.LogEntryFilter = null;

            string entryText = "LogTest3";
            string category = "LogTest3 category";
            bool expected = true;
            bool actual = logger.Log(entryText, category);
            Assert.AreEqual(expected, actual, "LogTest3");
        }

        /// <summary>
        /// Test Log entry, severity, category, no filter.
        /// </summary>

        private void LogTest4(ILogger<string> logger)
        {

            LoggerFilter<string> filter = new LoggerFilter<string>();
            logger.LogEntryFilter = null;

            string entryText = "LogTest4";
            LogEntrySeverity severity = LogEntrySeverity.Information;
            string category = "LogTest4 category";
            bool expected = true;
            bool actual = logger.Log(entryText, severity, category);
            Assert.AreEqual(expected, actual, "LogTest4");
        }

        /// <summary>
        /// Test Log entry, with filter.
        /// </summary>
        private void LogTest5(ILogger<string> logger)
        {

            LoggerFilter<string> filter = new LoggerFilter<string>();
            logger.LogEntryFilter = filter;

            string entryText = "LogTest5";
            bool expected = false;
            bool actual = logger.Log(entryText);
            Assert.AreEqual(expected, actual, "LogTest5");
        }

        /// <summary>
        /// Test Log entry, severity, with filter.
        /// </summary>

        private void LogTest6(ILogger<string> logger)
        {

            LoggerFilter<string> filter = new LoggerFilter<string>();
            logger.LogEntryFilter = filter;

            string entryText = "LogTest6";
            LogEntrySeverity severity = LogEntrySeverity.Information;
            bool expected = false;
            bool actual = logger.Log(entryText, severity);
            Assert.AreEqual(expected, actual, "LogTest6");
        }

        /// <summary>
        /// Test Log entry, category (no match), with filter.
        /// </summary>

        private void LogTest7(ILogger<string> logger)
        {

            LoggerFilter<string> filter = new LoggerFilter<string>();
            logger.LogEntryFilter = filter;

            string entryText = "LogTest7";
            string category = LoggerFilter<string>.CategoryFailedValue;
            bool expected = false;
            bool actual = logger.Log(entryText, category);
            Assert.AreEqual(expected, actual, "LogTest7");
        }

        /// <summary>
        /// Test Log entry, severity, category (no match), with filter.
        /// </summary>

        private void LogTest8(ILogger<string> logger)
        {

            LoggerFilter<string> filter = new LoggerFilter<string>();
            logger.LogEntryFilter = filter;

            string entryText = "LogTest8";
            LogEntrySeverity severity = LogEntrySeverity.Information;
            string category = LoggerFilter<string>.CategoryFailedValue;
            bool expected = false;
            bool actual = logger.Log(entryText, severity, category);
            Assert.AreEqual(expected, actual, "LogTest8");
        }

        /// <summary>
        /// Test Log entry, category (match), with filter.
        /// </summary>

        private void LogTest9(ILogger<string> logger)
        {

            LoggerFilter<string> filter = new LoggerFilter<string>();
            logger.LogEntryFilter = filter;

            string entryText = "LogTest9";
            string category = LoggerFilter<string>.CategoryPassedValue;
            bool expected = true;
            bool actual = logger.Log(entryText, category);
            Assert.AreEqual(expected, actual, "LogTest9");
        }

        /// <summary>
        /// Test Log entry, severity, category (match), with filter.
        /// </summary>

        private void LogTest10(ILogger<string> logger)
        {

            LoggerFilter<string> filter = new LoggerFilter<string>();
            logger.LogEntryFilter = filter;

            string entryText = "LogTest10";
            LogEntrySeverity severity = LogEntrySeverity.Information;
            string category = LoggerFilter<string>.CategoryPassedValue;
            bool expected = true;
            bool actual = logger.Log(entryText, severity, category);
            Assert.AreEqual(expected, actual, "LogTest10");
        }

 
    }
}
