﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using tpts.Logging;
using System.Diagnostics.Contracts;

namespace tptsLoggingUnitTests
{
    /// <summary>
    /// Class to allow testing of filtering for loggers.  Known value for category.
    /// </summary>
    /// <typeparam name="T"></typeparam>
    sealed class LoggerFilter<T> : ILogEntryFilter<T>
    {

        private const string CATGEORY_PASSED = "C";
        private const string CATGEORY_FAILED = "NC";

        public static string CategoryPassedValue
        {
            get { return CATGEORY_PASSED; }
        }

        public static string CategoryFailedValue
        {
            get { return CATGEORY_FAILED; }
        }

        #region Declarations

        #endregion

        #region Constructors

        /// <summary>
        /// Constructor for category filter, specifying the category to filter on.
        /// </summary>
        /// <param name="category"></param>
        /// <remarks>
        /// Contract: pre  : Category must not be null.
        /// </remarks>
        public LoggerFilter()
        {
        }

        #endregion

        #region Properties

        /// <summary>
        /// Get/Set the category for the filter.
        /// </summary>
        /// <remarks>
        /// Contract: pre  : Category must not be null.
        /// </remarks>
        public string Category
        {
            get { return CATGEORY_PASSED; }
        }

        #endregion

        #region Public Methods
        #endregion

        #region Protected Methods
        #endregion

        #region Private Methods
        #endregion

        #region ILogEntryFilter

        /// <summary>
        /// Return true if the log entry category equals the filter category based on the current culture settings.
        /// </summary>
        /// <param name="logEntry"></param>
        /// <returns>A boolean indicating if the log entry log entry category equals the filter category.</returns>
        /// <remarks>
        /// Note: If the filter category is blank then all log entries are classified as matching.
        /// 
        /// Contract: pre  : logEntry must not be null.
        /// </remarks>
        public bool Accept(LogEntry<T> logEntry)
        {
            return (CATGEORY_PASSED.Equals(logEntry.Category, StringComparison.CurrentCulture));
        }

        #endregion

    }
}
