﻿using tpts.Logging;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Diagnostics.Contracts;

namespace tptsLoggingUnitTests
{


    /// <summary>
    ///This is a test class for XMLFileLoggerTest and is intended
    ///to contain all XMLFileLoggerTest Unit Tests
    ///</summary>
    [TestClass()]
    public class XMLFileLoggerTest
    {

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion

        /// <summary>
        /// Constructor for XMLFileLoggerTest
        /// </summary>
        public XMLFileLoggerTest()
        {
            // Add event handler for contract failures, required so tests can trap any contract exceptions
            // Required to allow unit tests to test contracts are correct.
            Contract.ContractFailed += (sender, e) =>
            {
                e.SetHandled();
                e.SetUnwind();
                throw new ContractTestFailedException(e.FailureKind + ": " + e.Message);
            };
        }

        #region Logging Test

        /// <summary>
        ///A test for LogFileName - file name and extension.
        ///</summary>
        [TestMethod()]
        public void LogTest1()
        {
            string logFileName = "XMLLoggerTestLogTest1.xml";
            XMLFileLogger<string> xmlLogger = CreateXMLFileLogger<string>(logFileName);
            TestLogMethod(logFileName, xmlLogger);
        }

        /// <summary>
        ///A test for LogFileName - file name and no extension.
        ///</summary>
        [TestMethod()]
        public void LogTest2()
        {
            string logFileName = "XMLLoggerTestLogTest2";
            XMLFileLogger<string> xmlLogger = CreateXMLFileLogger<string>(logFileName);
            TestLogMethod(logFileName, xmlLogger);
        }

        /// <summary>
        ///A test for LogFileName - file name, path and  extension.
        ///</summary>
        [TestMethod()]
        public void LogTest3()
        {
            string logFileName = System.IO.Path.Combine(System.IO.Path.GetTempPath(), "LogTest3.xml");
            XMLFileLogger<string> xmlLogger = CreateXMLFileLogger<string>(logFileName);
            TestLogMethod(logFileName, xmlLogger);
        }

        /// <summary>
        ///A test for LogFileName - file name, path and no  extension.
        ///</summary>
        [TestMethod()]
        public void LogTest4()
        {
            string logFileName = System.IO.Path.Combine(System.IO.Path.GetTempPath(), "LogTest4");
            XMLFileLogger<string> xmlLogger = CreateXMLFileLogger<string>(logFileName);
            TestLogMethod(logFileName, xmlLogger);
        }

        #endregion

        #region Rollover Tests


        /// <summary>
        /// Test rolling of file name with no path and extension.
        /// </summary>
        [TestMethod()]
        public void RollOverTest1()
        {

            string logFileName = "RollOverTest1.txt";
            IFileRolloverStrategy rolloverStrategy = new FixedDateRolloverStrategy();
            XMLFileLogger<string> logger = CreateXMLFileLogger<string>(logFileName, rolloverStrategy);
            string expected = rolloverStrategy.GetRolledFileName(logFileName);
            string entryText = "RollOverTest1 Entry";
            if (System.IO.File.Exists(expected))
            {
                System.IO.File.Delete(expected);
            }
            Assert.IsFalse(System.IO.File.Exists(expected));
            logger.Log(entryText);
            Assert.IsTrue(System.IO.File.Exists(expected), "Log File '" + expected + "' was not found");
            if (System.IO.File.Exists(expected))
            {
                System.IO.File.Delete(expected);
            }
        }

        /// <summary>
        /// Test rolling of file name with path and extension.
        /// </summary>
        [TestMethod()]
        public void RollOverTest2()
        {

            string logFileName = System.IO.Path.Combine(System.IO.Path.GetTempPath(), "RollOverTest2.txt");
            IFileRolloverStrategy rolloverStrategy = new FixedDateRolloverStrategy();
            XMLFileLogger<string> logger = CreateXMLFileLogger<string>(logFileName, rolloverStrategy);
            string expected = rolloverStrategy.GetRolledFileName(logFileName);
            string entryText = "RollOverTest2 Entry";
            if (System.IO.File.Exists(expected))
            {
                System.IO.File.Delete(expected);
            }
            Assert.IsFalse(System.IO.File.Exists(expected));
            logger.Log(entryText);
            Assert.IsTrue(System.IO.File.Exists(expected), "Log File '" + expected + "' was not found");
            if (System.IO.File.Exists(expected))
            {
                System.IO.File.Delete(expected);
            }
        }

        /// <summary>
        /// Test rolling of file name with no path and no extension.
        /// </summary>
        [TestMethod()]
        public void RollOverTest3()
        {

            string logFileName = "RollOverTest3.txt";
            IFileRolloverStrategy rolloverStrategy = new FixedDateRolloverStrategy();
            XMLFileLogger<string> logger = CreateXMLFileLogger<string>(logFileName, rolloverStrategy);
            string expected = rolloverStrategy.GetRolledFileName(logFileName);
            string entryText = "RollOverTest3 Entry";
            if (System.IO.File.Exists(expected))
            {
                System.IO.File.Delete(expected);
            }
            Assert.IsFalse(System.IO.File.Exists(expected));
            logger.Log(entryText);
            Assert.IsTrue(System.IO.File.Exists(expected), "Log File '" + expected + "' was not found");
            if (System.IO.File.Exists(expected))
            {
                System.IO.File.Delete(expected);
            }
        }

        /// <summary>
        /// Test rolling of file name with path and no extension.
        /// </summary>
        [TestMethod()]
        public void RollOverTest4()
        {

            string logFileName = System.IO.Path.Combine(System.IO.Path.GetTempPath(), "RollOverTest4");
            IFileRolloverStrategy rolloverStrategy = new FixedDateRolloverStrategy();
            XMLFileLogger<string> logger = CreateXMLFileLogger<string>(logFileName, rolloverStrategy);
            string expected = rolloverStrategy.GetRolledFileName(logFileName);
            string entryText = "RollOverTest4 Entry";
            if (System.IO.File.Exists(expected))
            {
                System.IO.File.Delete(expected);
            }
            Assert.IsFalse(System.IO.File.Exists(expected));
            logger.Log(entryText);
            Assert.IsTrue(System.IO.File.Exists(expected), "Log File '" + expected + "' was not found");
            if (System.IO.File.Exists(expected))
            {
                System.IO.File.Delete(expected);
            }
        }

        #endregion


        #region Standard Logger Tests

        [TestMethod()]
        public void StandardLoggerTests()
        {
            ILoggerTestHelper loggerTests = new ILoggerTestHelper();
            string logFileName = "XMLLoggerTestStandardLoggerTests.xml";
            loggerTests.RunStandardTests(CreateXMLFileLogger<string>(logFileName));
        }


        #endregion

        #region Contract Tests

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest1()
        {
            try
            {
                string logFileName = "XMLLoggerTestLogNullEntryContractTest1.xml";
                XMLFileLogger<string> logger = CreateXMLFileLogger<string>(logFileName);
                string entryText = null;

                logger.Log(entryText);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest1", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest2()
        {
            try
            {
                string logFileName = "XMLLoggerTestLogNullEntryContractTest2.xml";
                XMLFileLogger<string> logger = CreateXMLFileLogger<string>(logFileName);
                string entryText = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;

                logger.Log(entryText, severity);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest2", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest3()
        {
            try
            {
                string logFileName = "XMLLoggerTestLogNullEntryContractTest3.xml";
                XMLFileLogger<string> logger = CreateXMLFileLogger<string>(logFileName);
                string entryText = null;
                string category = "LogNullEntryContractTest3 Category";

                logger.Log(entryText, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest3", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullEntryContractTest4()
        {
            try
            {
                string logFileName = "XMLLoggerTestLogNullEntryContractTest4.xml";
                XMLFileLogger<string> logger = CreateXMLFileLogger<string>(logFileName);
                string entryText = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;
                string category = "LogNullEntryContractTest4 Category";

                logger.Log(entryText, severity, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullEntryContractTest4", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null category
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullCategoryContractTest1()
        {
            try
            {
                string logFileName = "XMLLoggerTestLogNullCategoryContractTest1.xml";
                XMLFileLogger<string> logger = CreateXMLFileLogger<string>(logFileName);
                string entryText = "LogNullCategoryContractTest1 Entry";
                string category = null;

                logger.Log(entryText, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullCategoryContractTest1", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when logging null entry
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void LogNullCategoryContractTest2()
        {
            try
            {
                string logFileName = "XMLLoggerTestLogNullCategoryContractTest2.xml";
                XMLFileLogger<string> logger = CreateXMLFileLogger<string>(logFileName);
                string entryText = "LogNullCategoryContractTest2 Entry";
                string category = null;
                LogEntrySeverity severity = LogEntrySeverity.Debug;

                logger.Log(entryText, severity, category);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: LogNullCategoryContractTest2", ex);
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Test contract volilation when creating instance with null log file name.
        /// </summary>
        [TestMethod()]
        [ExpectedException(typeof(ContractTestFailedException))]
        public void ConstructorNullContractTest1()
        {
            try
            {
                string logFileName = null;
                XMLFileLogger<string> xmlLogger = new XMLFileLogger<string>(logFileName);

            }
            catch (Exception ex)
            {
                if (CommonFunctions.IsContractException(ex))
                {
                    throw new ContractTestFailedException("Failed contract: ConstructorNullContractTest1", ex);
                }
                else
                {
                    throw;
                }
            }
        }


        #endregion

        #region Private Methods

        private XMLFileLogger<T> CreateXMLFileLogger<T>(string logFileName)
        {
            return new XMLFileLogger<T>(logFileName);
        }

        private XMLFileLogger<T> CreateXMLFileLogger<T>(string logFileName,IFileRolloverStrategy rolloverStrategy)
        {
            return new XMLFileLogger<T>(logFileName, rolloverStrategy);
        }

        private string GetExpectedDataFileName(string logFileName)
        {
            string ext = System.IO.Path.GetExtension(logFileName);
            int len = logFileName.Length - ext.Length;
            if (0 < len) // len should always be greater than 1, here to satisfy contract for string.Substring.
            {
                return logFileName.Substring(0, len) + "_Entries" + ext;
            }
            else
            {
                return "_Entries" + ext;
            }
        }

        private void TestLogMethod(string logFileName, XMLFileLogger<string> xmlLogger)
        {
            FixedFormatter<string> formatter = new FixedFormatter<string>();
            xmlLogger.LogEntryFormatter = formatter;
            string expected = logFileName;
            string actual;
            actual = logFileName;

            expected = GetExpectedDataFileName(logFileName);

            if (System.IO.File.Exists(actual))
            {
                System.IO.File.Delete(actual);
            }
            if (System.IO.File.Exists(expected))
            {
                System.IO.File.Delete(expected);
            }
            Assert.AreEqual(false, System.IO.File.Exists(actual));
            Assert.AreEqual(false, System.IO.File.Exists(expected));
            xmlLogger.Log("LogTest1 Entry");
            Assert.AreEqual(true, System.IO.File.Exists(actual), "Header file does not exist " + actual);
            Assert.AreEqual(true, System.IO.File.Exists(expected), "Data file does not exist " + expected);
            if (System.IO.File.Exists(actual))
            {
                string contents = System.IO.File.ReadAllText(actual);
                Assert.IsTrue(contents.Contains("xml"));
            }
            if (System.IO.File.Exists(actual))
            {
                string contents = System.IO.File.ReadAllText(actual);
                Assert.IsTrue(contents.Contains("xml"));
                Assert.IsFalse(contents.Contains("LogTest1 Entry"));
            }
            if (System.IO.File.Exists(expected))
            {
                string contents = System.IO.File.ReadAllText(expected);
                Assert.IsTrue(contents.Contains("LogTest1 Entry"));
            }

        }
        #endregion


    }
}
